@@ -1,4 +1,7 @@
-#[derive(Clone, Debug)]
+use std::collections::HashMap;
+use std::sync::Arc;
+
+#[derive(Copy, Clone, Debug)]
 pub struct PriorityConfig {
     pub max_string_graphemes: usize,
     pub array_max_items: usize,
@@ -53,6 +56,7 @@ pub enum ObjectType {
 pub enum ArrayBias {
     Head,
     HeadMidTail,
+    HeadTail,
 }
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
@@ -135,6 +139,7 @@ pub struct NodeMetrics {
 pub struct PriorityOrder {
     pub metrics: Vec<NodeMetrics>,
     pub nodes: Vec<RankedNode>,
+    pub scores: Vec<u128>,
     // All ids in this structure are PQ ids (0..total_nodes).
     // They correspond to `NodeId.0` in `RankedNode` for convenience when indexing.
     pub parent: Vec<Option<NodeId>>, // parent[id] = parent id (PQ id)
@@ -145,6 +150,11 @@ pub struct PriorityOrder {
     pub by_priority: Vec<NodeId>, // ids sorted by ascending priority (PQ ids)
     pub total_nodes: usize,
     pub object_type: Vec<ObjectType>,
+    pub force_first_child: Vec<bool>,
+    pub code_lines: HashMap<usize, Arc<Vec<String>>>,
+    // For filesets, preserve the ingest order of top-level children so rendering
+    // can respect pre-sorting heuristics (e.g., frecency).
+    pub fileset_children: Option<Vec<NodeId>>,
 }
 
 pub const ROOT_PQ_ID: usize = 0;
