@@ -1,7 +1,10 @@
 use crate::order::ObjectType;
-use crate::order::{NodeKind, PriorityOrder, ROOT_PQ_ID, RankedNode};
+use crate::order::{NodeId, NodeKind, PriorityOrder, ROOT_PQ_ID, RankedNode};
+use std::collections::HashMap;
+use std::sync::Arc;
 pub mod color;
 mod fileset;
+mod highlight;
 pub mod output;
 pub mod templates;
 pub mod types;
@@ -22,9 +25,162 @@ pub(crate) struct RenderScope<'a> {
     render_set_id: u32,
     // Rendering configuration (template, whitespace, etc.).
     config: &'a crate::RenderConfig,
+    // Optional global width for line number alignment when enabled.
+    line_number_width: Option<usize>,
+    code_highlight_cache: HashMap<usize, Arc<Vec<String>>>,
 }
 
 impl<'a> RenderScope<'a> {
+    fn is_text_omission_line(
+        style: crate::serialization::types::Style,
+        raw: &str,
+    ) -> bool {
+        let s = raw.trim();
+        if s.is_empty() {
+            return true;
+        }
+        match style {
+            // Default: a single omission marker
+            crate::serialization::types::Style::Default => s == "…",
+            // Detailed: either single marker or "… N more lines …"
+            crate::serialization::types::Style::Detailed => {
+                if s == "…" {
+                    return true;
+                }
+                s.starts_with('…') && s.ends_with('…')
+            }
+            // Strict: arrays do not emit omission lines in text mode; treat empty-only as omission
+            crate::serialization::types::Style::Strict => s.is_empty(),
+        }
+    }
+
+    fn rendered_is_pure_text_omission(&self, rendered: &str) -> bool {
+        // Consider it pure omission if all non-empty lines match omission pattern for current style.
+        let mut any = false;
+        for line in rendered.split('\n') {
+            if line.trim().is_empty() {
+                continue;
+            }
+            any = true;
+            if !Self::is_text_omission_line(self.config.style, line) {
+                return false;
+            }
+        }
+        any
+    }
+
+    fn source_hint_for(&self, id: usize) -> Option<&'a str> {
+        let mut cursor = Some(NodeId(id));
+        while let Some(node) = cursor {
+            if let Some(key) = self.order.nodes[node.0].key_in_object() {
+                return Some(key);
+            }
+            cursor = self.order.parent.get(node.0).and_then(|parent| *parent);
+        }
+        self.config.primary_source_name.as_deref()
+    }
+    fn code_root_array_id(&self, array_id: usize) -> usize {
+        let mut current = array_id;
+        while let Some(Some(parent)) = self.order.parent.get(current) {
+            match self.order.nodes[parent.0] {
+                RankedNode::Array { .. } => current = parent.0,
+                _ => break,
+            }
+        }
+        current
+    }
+
+    fn push_code_line(
+        &self,
+        child_idx: usize,
+        text: &str,
+        acc: &mut Vec<Option<String>>,
+    ) {
+        let idx = self
+            .order
+            .index_in_parent_array
+            .get(child_idx)
+            .and_then(|o| *o)
+            .unwrap_or(0);
+        if acc.len() <= idx {
+            acc.resize(idx + 1, None);
+        }
+        acc[idx] = Some(text.to_string());
+    }
+
+    fn collect_code_lines(
+        &self,
+        array_id: usize,
+        acc: &mut Vec<Option<String>>,
+    ) {
+        if let Some(children) = self.order.children.get(array_id) {
+            for child in children {
+                let child_idx = child.0;
+                match &self.order.nodes[child_idx] {
+                    RankedNode::Array { .. } | RankedNode::Object { .. } => {
+                        self.collect_code_lines(child_idx, acc);
+                    }
+                    RankedNode::SplittableLeaf { value, .. } => {
+                        self.push_code_line(child_idx, value, acc);
+                    }
+                    RankedNode::AtomicLeaf { token, .. } => {
+                        self.push_code_line(child_idx, token, acc);
+                    }
+                    RankedNode::LeafPart { .. } => {}
+                }
+            }
+        }
+    }
+
+    fn compute_code_highlights(&self, array_id: usize) -> Vec<String> {
+        let root = self.code_root_array_id(array_id);
+        if let Some(full) = self.order.code_lines.get(&root) {
+            let mut highlighter =
+                crate::serialization::highlight::CodeHighlighter::new(
+                    self.source_hint_for(root),
+                );
+            return full
+                .iter()
+                .map(|line| highlighter.highlight_line(line))
+                .collect();
+        }
+        let mut lines: Vec<Option<String>> = Vec::new();
+        self.collect_code_lines(array_id, &mut lines);
+        if lines.is_empty() {
+            return Vec::new();
+        }
+        let mut highlighter =
+            crate::serialization::highlight::CodeHighlighter::new(
+                self.source_hint_for(array_id),
+            );
+        lines
+            .into_iter()
+            .map(|opt| {
+                let text = opt.unwrap_or_default();
+                highlighter.highlight_line(&text)
+            })
+            .collect()
+    }
+
+    fn code_highlights_for(
+        &mut self,
+        array_id: usize,
+        template: crate::OutputTemplate,
+    ) -> Option<Arc<Vec<String>>> {
+        if !self.config.color_enabled {
+            return None;
+        }
+        if !matches!(template, crate::OutputTemplate::Code) {
+            return None;
+        }
+        let root = self.code_root_array_id(array_id);
+        if let Some(existing) = self.code_highlight_cache.get(&root) {
+            return Some(existing.clone());
+        }
+        let computed = Arc::new(self.compute_code_highlights(root));
+        self.code_highlight_cache.insert(root, computed.clone());
+        Some(computed)
+    }
     fn push_array_child_line(
         &self,
         out: &mut Vec<ArrayChildPair>,
@@ -106,6 +262,8 @@ impl<'a> RenderScope<'a> {
             depth,
             inline_open: inline,
             omitted_at_start: config.prefer_tail_arrays,
+            source_hint: self.source_hint_for(id),
+            code_highlight: self.code_highlights_for(id, config.template),
         };
         render_array(config.template, &ctx, out)
     }
@@ -177,6 +335,7 @@ impl<'a> RenderScope<'a> {
         if matches!(
             self.config.template,
             crate::serialization::types::OutputTemplate::Text
+                | crate::serialization::types::OutputTemplate::Code
         ) {
             if omitted == 0 {
                 full.to_string()
@@ -227,6 +386,7 @@ impl<'a> RenderScope<'a> {
         if matches!(
             template,
             crate::serialization::types::OutputTemplate::Text
+                | crate::serialization::types::OutputTemplate::Code
         ) {
             if omitted == 0 {
                 full.to_string()
@@ -275,8 +435,9 @@ impl<'a> RenderScope<'a> {
                 if matches!(
                     self.config.template,
                     crate::serialization::types::OutputTemplate::Text
+                        | crate::serialization::types::OutputTemplate::Code
                 ) {
-                    // For text template, push raw string without quotes or color.
+                    // For text/code templates, push raw string without quotes or color.
                     out.push_str(&s);
                 } else {
                     out.push_string_literal(&s);
@@ -292,6 +453,10 @@ impl<'a> RenderScope<'a> {
         }
     }
 
+    #[allow(
+        clippy::cognitive_complexity,
+        reason = "Text omission filtering adds a branch; clearer inline"
+    )]
     fn gather_array_children(
         &mut self,
         id: usize,
@@ -304,10 +469,25 @@ impl<'a> RenderScope<'a> {
                 if self.inclusion_flags[child_id.0] != self.render_set_id {
                     continue;
                 }
-                kept += 1;
                 let child_kind = self.order.nodes[child_id.0].display_kind();
                 let rendered =
                     self.render_node_to_string(child_id.0, depth + 1, false);
+                // Text template: keep omission-only children so their own
+                // renderer can place markers at the correct indentation.
+                // Coalesce: if the previously kept child is also a pure
+                // omission, skip this one to avoid consecutive markers.
+                if matches!(
+                    self.config.template,
+                    crate::serialization::types::OutputTemplate::Text
+                ) && self.rendered_is_pure_text_omission(&rendered)
+                {
+                    if let Some((_pi, (_pk, prev_s))) = children_pairs.last() {
+                        if self.rendered_is_pure_text_omission(prev_s) {
+                            continue;
+                        }
+                    }
+                }
+                kept += 1;
                 let orig_index = self
                     .order
                     .index_in_parent_array
@@ -326,6 +506,10 @@ impl<'a> RenderScope<'a> {
         (children_pairs, kept)
     }
 
+    #[allow(
+        clippy::cognitive_complexity,
+        reason = "Text omission filtering adds a branch; clearer inline"
+    )]
     fn gather_array_children_with_template(
         &mut self,
         id: usize,
@@ -339,7 +523,6 @@ impl<'a> RenderScope<'a> {
                 if self.inclusion_flags[child_id.0] != self.render_set_id {
                     continue;
                 }
-                kept += 1;
                 let child_kind = self.order.nodes[child_id.0].display_kind();
                 let rendered = self.render_node_to_string_with_template(
                     child_id.0,
@@ -347,6 +530,18 @@ impl<'a> RenderScope<'a> {
                     false,
                     template,
                 );
+                if matches!(
+                    template,
+                    crate::serialization::types::OutputTemplate::Text
+                ) && self.rendered_is_pure_text_omission(&rendered)
+                {
+                    if let Some((_pi, (_pk, prev_s))) = children_pairs.last() {
+                        if self.rendered_is_pure_text_omission(prev_s) {
+                            continue;
+                        }
+                    }
+                }
+                kept += 1;
                 let orig_index = self
                     .order
                     .index_in_parent_array
@@ -427,25 +622,15 @@ impl<'a> RenderScope<'a> {
         match &self.order.nodes[id] {
             RankedNode::Array { .. } => {
                 let mut s = String::new();
-                let mut ow = Out::new(
-                    &mut s,
-                    &self.config.newline,
-                    &self.config.indent_unit,
-                    self.config.color_enabled,
-                    self.config.style,
-                );
+                let mut ow =
+                    Out::new(&mut s, self.config, self.line_number_width);
                 self.write_array(id, depth, inline, &mut ow);
                 s
             }
             RankedNode::Object { .. } => {
                 let mut s = String::new();
-                let mut ow = Out::new(
-                    &mut s,
-                    &self.config.newline,
-                    &self.config.indent_unit,
-                    self.config.color_enabled,
-                    self.config.style,
-                );
+                let mut ow =
+                    Out::new(&mut s, self.config, self.line_number_width);
                 self.write_object(id, depth, inline, &mut ow);
                 s
             }
@@ -478,6 +663,8 @@ impl<'a> RenderScope<'a> {
             depth,
             inline_open: inline,
             omitted_at_start: config.prefer_tail_arrays,
+            source_hint: self.source_hint_for(id),
+            code_highlight: self.code_highlights_for(id, template),
         };
         render_array(template, &ctx, out)
     }
@@ -519,13 +706,8 @@ impl<'a> RenderScope<'a> {
         match &self.order.nodes[id] {
             RankedNode::Array { .. } => {
                 let mut s = String::new();
-                let mut ow = Out::new(
-                    &mut s,
-                    &self.config.newline,
-                    &self.config.indent_unit,
-                    self.config.color_enabled,
-                    self.config.style,
-                );
+                let mut ow =
+                    Out::new(&mut s, self.config, self.line_number_width);
                 self.write_array_with_template(
                     id, depth, inline, &mut ow, template,
                 );
@@ -533,13 +715,8 @@ impl<'a> RenderScope<'a> {
             }
             RankedNode::Object { .. } => {
                 let mut s = String::new();
-                let mut ow = Out::new(
-                    &mut s,
-                    &self.config.newline,
-                    &self.config.indent_unit,
-                    self.config.color_enabled,
-                    self.config.style,
-                );
+                let mut ow =
+                    Out::new(&mut s, self.config, self.line_number_width);
                 self.write_object_with_template(
                     id, depth, inline, &mut ow, template,
                 );
@@ -558,6 +735,35 @@ impl<'a> RenderScope<'a> {
 
 /// Prepare a render set by including the first `top_k` nodes by priority
 /// and all of their ancestors so the output remains structurally valid.
+fn enforce_force_first_child(
+    order_build: &PriorityOrder,
+    inclusion_flags: &mut [u32],
+    render_id: u32,
+) {
+    for (idx, force) in order_build.force_first_child.iter().enumerate() {
+        let included =
+            inclusion_flags.get(idx).copied().unwrap_or_default() == render_id;
+        if !*force || !included {
+            continue;
+        }
+        let Some(children) = order_build.children.get(idx) else {
+            continue;
+        };
+        let Some(first_child) = children.first() else {
+            continue;
+        };
+        if inclusion_flags[first_child.0] == render_id {
+            continue;
+        }
+        crate::utils::graph::mark_node_and_ancestors(
+            order_build,
+            *first_child,
+            inclusion_flags,
+            render_id,
+        );
+    }
+}
+
 pub fn prepare_render_set_top_k_and_ancestors(
     order_build: &PriorityOrder,
     top_k: usize,
@@ -574,6 +780,7 @@ pub fn prepare_render_set_top_k_and_ancestors(
         inclusion_flags,
         render_id,
     );
+    enforce_force_first_child(order_build, inclusion_flags, render_id);
 }
 
 /// Render using a previously prepared render set (inclusion flags matching `render_id`).
@@ -584,20 +791,87 @@ pub fn render_from_render_set(
     config: &crate::RenderConfig,
 ) -> String {
     let root_id = ROOT_PQ_ID;
+    // Compute optional global line-number width when numbering is enabled for text.
+    fn digits(mut n: usize) -> usize {
+        if n == 0 {
+            return 1;
+        }
+        let mut d = 0;
+        while n > 0 {
+            d += 1;
+            n /= 10;
+        }
+        d
+    }
+    fn max_index_for_child(
+        order: &PriorityOrder,
+        flags: &[u32],
+        rid: u32,
+        child: usize,
+    ) -> Option<usize> {
+        if flags.get(child).copied().unwrap_or_default() != rid {
+            return None;
+        }
+        match order.nodes[child] {
+            RankedNode::AtomicLeaf { .. }
+            | RankedNode::SplittableLeaf { .. } => {
+                order.index_in_parent_array.get(child).and_then(|idx| *idx)
+            }
+            RankedNode::Array { .. } | RankedNode::Object { .. } => {
+                Some(compute_max_index(order, flags, rid, child))
+            }
+            _ => None,
+        }
+    }
+
+    fn compute_max_index(
+        order: &PriorityOrder,
+        flags: &[u32],
+        rid: u32,
+        id: usize,
+    ) -> usize {
+        let mut max_idx = 0usize;
+        if let Some(kids) = order.children.get(id) {
+            for &cid in kids.iter() {
+                let child_id = cid.0;
+                if let Some(child_max) =
+                    max_index_for_child(order, flags, rid, child_id)
+                {
+                    if child_max > max_idx {
+                        max_idx = child_max;
+                    }
+                }
+            }
+        }
+        max_idx
+    }
+    let root_is_fileset =
+        order_build.object_type.get(root_id) == Some(&ObjectType::Fileset);
+    let should_measure_line_numbers =
+        matches!(config.template, crate::OutputTemplate::Code)
+            || (matches!(config.template, crate::OutputTemplate::Auto)
+                && root_is_fileset);
+    let line_number_width = if should_measure_line_numbers {
+        let max_index = compute_max_index(
+            order_build,
+            inclusion_flags,
+            render_id,
+            root_id,
+        );
+        Some(digits(max_index.saturating_add(1)))
+    } else {
+        None
+    };
     let mut scope = RenderScope {
         order: order_build,
         inclusion_flags,
         render_set_id: render_id,
         config,
+        line_number_width,
+        code_highlight_cache: HashMap::new(),
     };
     let mut s = String::new();
-    let mut out = Out::new(
-        &mut s,
-        &config.newline,
-        &config.indent_unit,
-        config.color_enabled,
-        config.style,
-    );
+    let mut out = Out::new(&mut s, config, line_number_width);
     scope.write_node(root_id, 0, false, &mut out);
     s
 }
@@ -660,6 +934,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("arena_render_empty", out);
@@ -696,6 +973,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         // Sanity: output should contain CRLF newlines and render the object child across lines.
@@ -734,6 +1014,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("arena_render_single", out);
@@ -773,6 +1056,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("array_omitted_pseudo_head", out_head);
@@ -793,6 +1079,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("array_omitted_pseudo_tail", out_tail);
@@ -830,6 +1119,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("array_omitted_js_head", out_head);
@@ -849,6 +1141,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("array_omitted_js_tail", out_tail);
@@ -886,6 +1181,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out_head);
@@ -906,6 +1204,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out_tail);
@@ -940,6 +1241,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out);
@@ -974,6 +1278,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out);
@@ -1006,6 +1313,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out);
@@ -1016,13 +1326,12 @@ mod tests {
     fn array_internal_gaps_yaml() {
         let ctx = mk_gap_ctx();
         let mut s = String::new();
-        let mut outw = crate::serialization::output::Out::new(
-            &mut s,
-            "\n",
-            "  ",
-            false,
+        let cfg = test_render_cfg(
+            crate::OutputTemplate::Yaml,
             crate::serialization::types::Style::Default,
         );
+        let mut outw =
+            crate::serialization::output::Out::new(&mut s, &cfg, None);
         super::templates::render_array(
             crate::OutputTemplate::Yaml,
             &ctx,
@@ -1067,6 +1376,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out);
@@ -1138,6 +1450,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         // Expect the first 5 characters plus an ellipsis, as a valid JSON string literal.
@@ -1172,6 +1487,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Default,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_yaml_valid(&out);
@@ -1213,12 +1531,17 @@ mod tests {
             color_enabled: false,
             style: crate::serialization::types::Style::Strict,
             string_free_prefix_graphemes: None,
+            debug: false,
+            primary_source_name: None,
+            show_fileset_headers: true,
         };
         let scope = RenderScope {
             order: &build,
             inclusion_flags: &marks,
             render_set_id: render_id,
             config: &cfg,
+            line_number_width: None,
+            code_highlight_cache: HashMap::new(),
         };
         // Atomic leaves never report omitted counts.
         let none = scope.omitted_for(crate::order::ROOT_PQ_ID, 0);
@@ -1251,6 +1574,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Strict,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         assert_snapshot!("inline_open_array_in_object_json", out);
@@ -1286,6 +1612,9 @@ mod tests {
                 color_enabled: false,
                 style: crate::serialization::types::Style::Detailed,
                 string_free_prefix_graphemes: None,
+                debug: false,
+                primary_source_name: None,
+                show_fileset_headers: true,
             },
         );
         // Should be a valid JS object with one property and an omitted summary.
@@ -1301,7 +1630,7 @@ mod tests {
         );
     }
 
-    fn mk_gap_ctx() -> super::templates::ArrayCtx {
+    fn mk_gap_ctx() -> super::templates::ArrayCtx<'static> {
         super::templates::ArrayCtx {
             children: vec![
                 (0, (crate::order::NodeKind::Number, "1".to_string())),
@@ -1313,6 +1642,8 @@ mod tests {
             depth: 0,
             inline_open: false,
             omitted_at_start: false,
+            source_hint: None,
+            code_highlight: None,
         }
     }
 
@@ -1320,17 +1651,36 @@ mod tests {
         needles.iter().for_each(|n| assert!(out.contains(n)));
     }
 
+    fn test_render_cfg(
+        template: crate::OutputTemplate,
+        style: crate::serialization::types::Style,
+    ) -> crate::RenderConfig {
+        crate::RenderConfig {
+            template,
+            indent_unit: "  ".to_string(),
+            space: " ".to_string(),
+            newline: "\n".to_string(),
+            prefer_tail_arrays: false,
+            color_mode: crate::ColorMode::Off,
+            color_enabled: false,
+            style,
+            string_free_prefix_graphemes: None,
+            debug: false,
+            primary_source_name: None,
+            show_fileset_headers: true,
+        }
+    }
+
     #[test]
     fn array_internal_gaps_pseudo() {
         let ctx = mk_gap_ctx();
         let mut s = String::new();
-        let mut outw = crate::serialization::output::Out::new(
-            &mut s,
-            "\n",
-            "  ",
-            false,
+        let cfg = test_render_cfg(
+            crate::OutputTemplate::Pseudo,
             crate::serialization::types::Style::Default,
         );
+        let mut outw =
+            crate::serialization::output::Out::new(&mut s, &cfg, None);
         super::templates::render_array(
             crate::OutputTemplate::Pseudo,
             &ctx,
@@ -1347,13 +1697,12 @@ mod tests {
     fn array_internal_gaps_js() {
         let ctx = mk_gap_ctx();
         let mut s = String::new();
-        let mut outw = crate::serialization::output::Out::new(
-            &mut s,
-            "\n",
-            "  ",
-            false,
+        let cfg = test_render_cfg(
+            crate::OutputTemplate::Js,
             crate::serialization::types::Style::Default,
         );
+        let mut outw =
+            crate::serialization::output::Out::new(&mut s, &cfg, None);
         super::templates::render_array(
             crate::OutputTemplate::Js,
             &ctx,
