@@ -1,7 +1,9 @@
 use crate::OutputTemplate;
 use crate::order::NodeKind;
 use crate::serialization::output::Out;
+use std::sync::Arc;
 
+mod code;
 mod core;
 mod js;
 mod json;
@@ -9,13 +11,15 @@ mod pseudo;
 mod text;
 mod yaml;
 //
-pub struct ArrayCtx {
+pub struct ArrayCtx<'a> {
     pub children: Vec<(usize, (NodeKind, String))>,
     pub children_len: usize,
     pub omitted: usize,
     pub depth: usize,
     pub inline_open: bool,
     pub omitted_at_start: bool,
+    pub source_hint: Option<&'a str>,
+    pub code_highlight: Option<Arc<Vec<String>>>,
 }
 
 pub struct ObjectCtx<'a> {
@@ -33,7 +37,7 @@ pub struct ObjectCtx<'a> {
 
 pub fn render_array(
     template: OutputTemplate,
-    ctx: &ArrayCtx,
+    ctx: &ArrayCtx<'_>,
     out: &mut Out<'_>,
 ) {
     match template {
@@ -44,6 +48,7 @@ pub fn render_array(
         OutputTemplate::Js => js::render_array(ctx, out),
         OutputTemplate::Yaml => yaml::render_array(ctx, out),
         OutputTemplate::Text => text::render_array(ctx, out),
+        OutputTemplate::Code => code::render_array(ctx, out),
     }
 }
 
@@ -60,5 +65,6 @@ pub fn render_object(
         OutputTemplate::Js => js::render_object(ctx, out),
         OutputTemplate::Yaml => yaml::render_object(ctx, out),
         OutputTemplate::Text => text::render_object(ctx, out),
+        OutputTemplate::Code => code::render_object(ctx, out),
     }
 }
